import 'dart:async';

import 'package:adhara/config.dart';
import 'package:adhara/datainterface/bean.dart';
import 'package:adhara/datainterface/storage/storage_provider.dart';
import 'package:adhara/datainterface/storage/storage_fields.dart';
import 'package:sqflite/sqflite.dart';

abstract class BeanStorageProvider extends StorageProvider {
  BeanStorageProvider([Config config]) : super(config);

  String get fieldsStringSchema {
    String schema = super.fieldsStringSchema;
    schema += ", ${Bean.CREATED_TIME} integer non null";
    schema += ", ${Bean.LAST_UPDATED_TIME} integer non null";
    return schema;
  }

  List<String> get selectColumns {
    List<String> columns = super.selectColumns;
    columns.add(Bean.CREATED_TIME);
    columns.add(Bean.LAST_UPDATED_TIME);
    return columns;
  }

  List<StorageField> get defaultFields{
    List<StorageField> _df = super.defaultFields;
    _df.addAll([
      NumericField(Bean.CREATED_TIME, nullable: false),
      NumericField(Bean.LAST_UPDATED_TIME, nullable: false)
    ]);
    return _df;
  }

  Future<Bean> insertBean(Bean bean) async {
    if (bean.createdTime == null) {
      bean.setCreatedTime();
    }
    int pk = await this.rawInsert(bean.toSerializableMap());
    bean.setLocalId(pk);
    return bean;
  }

  Future<List<Bean>> insertBeans(List<Bean> beans) async {
    List<Map<String, dynamic>> entries = [];
    beans.forEach((Bean bean) {
      if (bean.createdTime == null) {
        bean.setCreatedTime();
      }
      entries.add(bean.toSerializableMap());
    });
    List<dynamic> results = await rawBulkInsert(entries);
    for (int i = 0; i < results.length; i++) {
      beans[i].setLocalId(results[i]);
    }
    return beans;
  }

  Future<int> updateBean(Bean bean) async {
    bean.setUpdatedTime();
    Map<String, dynamic> sdMap = bean.toSerializableMap();
    return await this.update(sdMap, "$idFieldName=?", [bean.identifier]);
  }

  Future<List<int>> updateBeans(List<Bean> beans) async { //TODO try to fit this method in storage_provider
    Batch batch = db.batch();
    beans.forEach((Bean bean) {
      bean.setUpdatedTime();
      Map<String, dynamic> sdMap = bean.toSerializableMap();
      batch.update(this.tableName, serialize(sdMap),
        where: "$idFieldName=?", whereArgs: [bean.identifier]);
      batch.update(this.tableName, bean.toSerializableMap());
    });
    List<dynamic> results = await batch.commit();
    for (int i = 0; i < results.length; i++) {
      beans[i].setLocalId(results[i]);
    }
    return results;
  }

  Future deleteBean(Bean bean) async {
    return await db
      .delete(this.tableName, where: "$idFieldName=?", whereArgs: [bean.identifier]);
  }
}
